@@ -1,10 +1,13 @@
-#ifndef READXL_XLSWORKBOOK_
-#define READXL_XLSWORKBOOK_
+#pragma once
 
-#include <Rcpp.h>
-#include <libxls/xls.h>
 #include "ColSpec.h"
-#include "utils.h"
+
+#include "libxls/xls.h"
+#include "libxls/xlsstruct.h"
+
+#include "cpp11/R.hpp"
+#include "cpp11/r_string.hpp"
+#include "cpp11/strings.hpp"
 
 class XlsWorkBook {
 
@@ -12,28 +15,37 @@ class XlsWorkBook {
   std::string path_;
   bool is1904_;
   std::set<int> dateFormats_;
+  std::vector<std::string> stringTable_;
 
   // kept as data + accessor in XlsWorkBook vs. member function in XlsxWorkBook
   int n_sheets_;
-  Rcpp::CharacterVector sheets_;
+  cpp11::writable::strings sheets_;
 
 public:
 
-  XlsWorkBook(const std::string& path) {
-    path_ = path;
+  XlsWorkBook(const std::string& path):
+  stringTable_{"placeholder"}
+  {
+    // the user's path has probably been translated to UTF-8 by
+    // normalizePath() on the R side
+    // even if that were not true, cpp11 does this automatically when
+    // constructing a std::string
+    // but we need to pass the path to libxls in the native encoding
+    path_ = std::string(Rf_translateChar(cpp11::r_string(path)));
 
     xls::xls_error_t error = xls::LIBXLS_OK;
     xls::xlsWorkBook* pWB_ = xls::xls_open_file(path_.c_str(), "UTF-8", &error);
     if (!pWB_) {
-      Rcpp::stop(
+      Rf_errorcall(
+        R_NilValue,
         "\n  filepath: %s\n  libxls error: %s",
-        path_,
+        path_.c_str(),
         xls::xls_getError(error)
       );
     }
 
     n_sheets_ = pWB_->sheets.count;
-    sheets_ = Rcpp::CharacterVector(n_sheets());
+    sheets_ = cpp11::writable::strings(n_sheets());
     for (int i = 0; i < n_sheets_; ++i) {
       sheets_[i] = Rf_mkCharCE((char*) pWB_->sheets.sheet[i].name, CE_UTF8);
     }
@@ -52,7 +64,7 @@ public:
     return n_sheets_;
   }
 
-  Rcpp::CharacterVector sheets() const {
+  cpp11::strings sheets() const {
     return sheets_;
   }
 
@@ -64,6 +76,10 @@ public:
     return dateFormats_;
   }
 
+  const std::vector<std::string>& stringTable() const {
+    return stringTable_;
+  }
+
 private:
 
   void cacheDateFormats(xls::xlsWorkBook* pWB) {
@@ -75,6 +91,9 @@ private:
     if (n_formats > 0) {
       for (int i = 0; i < n_formats; ++i) {
         const xls::st_format::st_format_data format = pWB->formats.format[i];
+        if (format.value == nullptr) {
+          continue;
+        }
         // format.value = format string
         // in xlsx, this is formatCode
         std::string code((char*) format.value);
@@ -103,5 +122,3 @@ private:
   }
 
 };
-
-#endif
